<?php
/**
 * WAAVE Compliance
 *
 * @class       WAAVE_Compliance
 * @package WAAVE_Compliance
 */

/**
 * WAAVE_Compliance
 */
class WAAVE_Compliance {
	const API_KEY_GOOGLE_FONT = 'AIzaSyAZnNr3PX2QGG349vh1CXlZ-H9fND82dbo';

	/**
	 * Instance.
	 *
	 * @var this
	 */
	public static $instance;

	/**
	 * Function init.
	 */
	public static function init() {
		if ( ! self::$instance ) {
			self::$instance = new WAAVE_Compliance();
		}

		return self::$instance;
	}

	/**
	 * Function construct.
	 */
	public function __construct() {
		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_tab' ), 99 );
		add_action( 'woocommerce_settings_waave', array( $this, 'settings_tab' ) );
		add_action( 'woocommerce_update_options_waave', array( $this, 'update_settings' ) );
		$plugin_data = get_plugin_data( WAAVE_COMPLIANCE_MAIN_FILE );
		$version     = $plugin_data['Version'];
		wp_enqueue_script('age-gate-configuration', plugins_url( '../assets/js/age-gate-configuration.js', __FILE__ ), array('jquery'), $version );
		wp_enqueue_style( 'age-gate-configuration', plugins_url( '../assets/css/age-gate-configuration.css', __FILE__ ), array(), $version );
		wp_localize_script(
			'age-gate-configuration',
			'age_gate_config',
			array(
				'ageGatePopupDesignMode' => get_option( 'waave_compliance_age_gate_popup_design_mode' ) ,
				'ageGatePopupSize' => get_option( 'waave_compliance_age_gate_popup_size' ),
				'ageGatePopupButtonColorDefault' => get_option( 'waave_compliance_age_gate_popup_button_color_default' ),
				'ageGatePopupLogoDefault' => get_option( 'waave_compliance_age_gate_popup_logo_default' ),
				'ageGatePopupFontFamilyType' => get_option( 'waave_compliance_age_gate_popup_font_family_type' ),
				'ageGatePopupMode' => get_option( 'waave_compliance_age_gate_mode' ),
			)
		);
	}

	/**
	 * Function add_settings_tab.
	 *
	 * @param array $settings_tabs settings_tabs.
	 */
	public function add_settings_tab( $settings_tabs ) {
		$settings_tabs['waave'] = 'WAAVE';
		return $settings_tabs;
	}

	/**
	 * Function settings_tabs.
	 */
	public function settings_tab() {
		woocommerce_admin_fields( $this->get_settings() );
	}

	/**
	 * Function update_settings.
	 */
	public function update_settings() {
		woocommerce_update_options( $this->get_settings() );
		$plugin_data = get_plugin_data( WAAVE_COMPLIANCE_MAIN_FILE );
		$version     = $plugin_data['Version'];
		wp_enqueue_script('age-gate-configuration', plugins_url( '../assets/js/age-gate-configuration.js', __FILE__ ), array('jquery'), $version );
		wp_enqueue_style( 'age-gate-configuration', plugins_url( '../assets/css/age-gate-configuration.css', __FILE__ ), array(), $version );
		wp_localize_script(
			'age-gate-configuration',
			'age_gate_config',
			array(
				'ageGatePopupDesignMode' => get_option( 'waave_compliance_age_gate_popup_design_mode' ) ,
				'ageGatePopupSize' => get_option( 'waave_compliance_age_gate_popup_size' ),
				'ageGatePopupButtonColorDefault' => get_option( 'waave_compliance_age_gate_popup_button_color_default' ),
				'ageGatePopupLogoDefault' => get_option( 'waave_compliance_age_gate_popup_logo_default' ),
				'ageGatePopupFontFamilyType' => get_option( 'waave_compliance_age_gate_popup_font_family_type' ),
				'ageGatePopupMode' => get_option( 'waave_compliance_age_gate_mode' ),
			)
		);
		$this->call_waave_active_api();
		$time_remerber_me = 10 * 365 * 24 * 60 * 60; // 10 years
		setcookie( 'age_verification_mode', get_option( 'waave_compliance_age_gate_mode' ) === 'yes' ? 1 : 0, time() + $time_remerber_me, COOKIEPATH, COOKIE_DOMAIN );
	}

	/**
	 * Function get_settings.
	 */
	private function get_settings() {
		$googleFontApiUrl = 'https://www.googleapis.com/webfonts/v1/webfonts?key=' . self::API_KEY_GOOGLE_FONT;
		$request = wp_remote_get($googleFontApiUrl);
		$response = json_decode( wp_remote_retrieve_body( $request ), true );
		$googleFonts = [];
		foreach ($response['items'] as $font) {
			$googleFonts[$font['family']] = $font['family'];
		}

		return array(
			'section_title'                => array(
				'name' => 'REST API',
				'type' => 'title',
				'desc' => '',
				'id'   => 'waave_compliance_section_title',
			),
			'consumer_key'                 => array(
				'name' => 'Consumer key',
				'type' => 'text',
				'desc' => '',
				'id'   => 'waave_compliance_consumer_key',
			),
			'consumer_secret'              => array(
				'name' => 'Consumer secret',
				'type' => 'password',
				'desc' => '',
				'id'   => 'waave_compliance_consumer_secret',
			),
			'section_end'                  => array(
				'type' => 'sectionend',
				'id'   => 'waave_compliance_section_end',
			),
			'section_client_title'         => array(
				'name' => 'Client',
				'type' => 'title',
				'desc' => '',
				'id'   => 'waave_compliance_section_client_title',
			),
			'testmode'                     => array(
				'name'    => 'WAAVE Sandbox',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_testmode',
				'default' => 'yes',
			),
			'venue_id'                     => array(
				'name' => 'Venue ID',
				'type' => 'text',
				'desc' => '',
				'id'   => 'waave_compliance_venue_id',
			),
			'client_password'              => array(
				'name' => 'Password',
				'type' => 'password',
				'desc' => '',
				'id'   => 'waave_compliance_client_password',
			),
			'enable_for_methods'           => array(
				'title'             => 'Enable for payment methods',
				'type'              => 'multiselect',
				'class'             => 'wc-enhanced-select',
				'css'               => 'width: 400px;',
				'default'           => '',
				'description'       => 'Enable for payment methods',
				'options'           => $this->load_payment_method_options(),
				'desc_tip'          => true,
				'custom_attributes' => array(
					'data-placeholder' => 'Select payment methods',
				),
				'id'                => 'waave_compliance_enabled_methods',
			),
			'section_client_end'           => array(
				'type' => 'sectionend',
				'id'   => 'waave_compliance_section_client_end',
			),
			'section_age_gate_title'       => array(
				'name' => 'Age Gate & Disclaimers',
				'type' => 'title',
				'desc' => '',
				'id'   => 'waave_compliance_age_gate_title',
			),
			'age_gate_mode'                => array(
				'name'    => 'WAAVE Age Gate Popup Mode',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_age_gate_mode',
				'desc'    => 'Enable/Disable',
				'default' => 'yes',
			),
			'disclaimers_mode'             => array(
				'name'    => 'WAAVE Disclaimers Mode',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_disclaimers_mode',
				'desc'    => 'Enable/Disable',
				'default' => 'yes',
			),
			'footer_mode'                  => array(
				'name'    => 'WAAVE Footer Mode',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_footer_mode',
				'desc'    => 'Enable/Disable',
				'default' => 'yes',
			),
			'disclaimers_background_color' => array(
				'name'    => '',
				'type'    => 'color',
				'id'      => 'waave_compliance_disclaimers_background_color',
				'desc'    => 'Background Waave Footer Color',
				'default' => '#fff',
				'class'   => 'inputcolor ',
			),
			'disclaimers_color'            => array(
				'name'    => '',
				'type'    => 'color',
				'id'      => 'waave_compliance_disclaimers_color',
				'desc'    => 'Waave Footer Color',
				'default' => '#111',
				'class'   => 'inputcolor ',
			),
			'age_gate_popup_design_mode'    => array(
				'name'    => 'WAAVE Age Gate Popup Design Mode',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_age_gate_popup_design_mode',
				'desc'    => 'Enable/Disable',
				'default' => 'no',
			),
			'age_gate_popup_logo_default'  => array(
				'name'    => '',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_age_gate_popup_logo_default',
				'desc'    => 'Default Logo',
				'default' => 'yes',
			),
			'age_gate_popup_logo'       => array(
				'name'     => 'WAAVE Age Gate Popup Logo',
				'type'     => 'url',
				'desc'     => '',
				'id'       => 'waave_compliance_age_gate_popup_logo',
				'desc_tip' => 'Paste the URL of an image you want to show in the age gate popup. Upload images using the media uploader (Media > Add New).'
			),
			'age_gate_popup_logo_width'       => array(
				'name'    => 'WAAVE Age Gate Popup Logo Width (px)',
				'type'    => 'number',
				'desc'    => '',
				'id'      => 'waave_compliance_age_gate_popup_logo_width',
				'default' => 350,
			),
			'age_gate_popup_logo_height'       => array(
				'name'    => 'WAAVE Age Gate Popup Logo Height (px)',
				'type'    => 'number',
				'desc'    => '',
				'id'      => 'waave_compliance_age_gate_popup_logo_height',
				'default' => 350,
			),
			'age_gate_popup_size'    => array(
				'name'    => 'WAAVE Age Gate Popup Size',
				'type'    => 'select',
				'id'      => 'waave_compliance_age_gate_popup_size',
				'desc'    => '',
				'default' => '',
				'options' => array(
					'large'  => 'Large',
					'medium' => 'Medium',
					'small'  => 'Small',
					'customize'  => 'Customize',
            	),
			),
			'age_gate_popup_width'       => array(
				'name' => 'WAAVE Age Gate Popup Width (px)',
				'type' => 'number',
				'desc' => '',
				'id'   => 'waave_compliance_age_gate_popup_width',
			),
			'age_gate_popup_height'       => array(
				'name' => 'WAAVE Age Gate Popup Height (px)',
				'type' => 'number',
				'desc' => '',
				'id'   => 'waave_compliance_age_gate_popup_height',
			),
			'age_gate_popup_background_color' => array(
				'name'    => '',
				'type'    => 'color',
				'id'      => 'waave_compliance_age_gate_popup_background_color',
				'desc'    => 'WAAVE Age Gate Popup Background Color',
				'default' => '#ffffff',
				'class'   => 'inputcolor ',
			),
			'age_gate_popup_color' => array(
				'name'    => '',
				'type'    => 'color',
				'id'      => 'waave_compliance_age_gate_popup_color',
				'desc'    => 'WAAVE Age Gate Popup Font Color',
				'default' => '#000000',
				'class'   => 'inputcolor ',
			),
			'age_gate_popup_font_size'    => array(
				'name'    => 'WAAVE Age Gate Popup Font Size',
				'type'    => 'select',
				'id'      => 'waave_compliance_age_gate_popup_font_size',
				'desc'    => '',
				'default' => '',
				'options' => array(
					'8' => 8,
					'9' => 9,
					'10' => 10,
					'11' => 11,
					'12' => 12,
					'14' => 14,
					'16' => 16,
					'18' => 18,
					'20' => 20,
					'22' => 22,
					'24' => 24,
					'26' => 26,
					'28' => 28,
					'36' => 36,
					'48' => 48,
					'72' => 72,
            	),
			),
			'age_gate_popup_font_family_type'    => array(
				'name'    => 'WAAVE Age Gate Popup Font Family Type',
				'type'    => 'select',
				'id'      => 'waave_compliance_age_gate_popup_font_family_type',
				'desc'    => '',
				'default' => '',
				'options' => array(
					'default' => 'Default',
					'google_font' => 'Google Font',
            	),
			),
			'age_gate_popup_google_font'    => array(
				'name'     => 'WAAVE Age Gate Popup Font Family',
				'type'     => 'select',
				'id'       => 'waave_compliance_age_gate_popup_google_font',
				'desc'     => '',
				'default'  => '',
				'options'  => $googleFonts,
			),
			'age_gate_popup_font_family'    => array(
				'name'     => 'WAAVE Age Gate Popup Font Family',
				'type'     => 'text',
				'desc'     => '',
				'id'       => 'waave_compliance_age_gate_popup_font_family',
			),
			'age_gate_popup_button_color_default'  => array(
				'name'    => '',
				'type'    => 'checkbox',
				'id'      => 'waave_compliance_age_gate_popup_button_color_default',
				'desc'    => 'Default Button Color',
				'default' => 'yes',
			),
			'age_gate_popup_button_color' => array(
				'name'    => '',
				'type'    => 'color',
				'id'      => 'waave_compliance_age_gate_popup_button_color',
				'desc'    => 'WAAVE Age Gate Popup Button Color',
				'default' => '#000000',
				'class'   => 'inputcolor ',
			),
			'section_age_gate_end'         => array(
				'type' => 'sectionend',
				'id'   => 'waave_compliance_age_gate_section_end',
			),
		);
	}

	/**
	 * Function load payment method options.
	 */
	private function load_payment_method_options() {
		$gateways = WC()->payment_gateways->get_available_payment_gateways();

		$options = array();
		foreach ( $gateways as $key => $gateway ) {
			$options[ $key ] = $gateway->title;
		}

		return $options;
	}

	/**
	 * WAAVE enabled payment methods.
	 *
	 * @param array $methods Enabled payment methods.
	 * @return array
	 */
	private function waave_enabled_payment_methods( $methods ) {
		$gateways = WC()->payment_gateways->get_available_payment_gateways();

		$options = array();
		foreach ( $gateways as $key => $gateway ) {
			if ( in_array( $key, $methods, true ) ) {
				$options[ $key ] = $gateway->title;
			}
		}

		return $options;
	}

	/**
	 * Function call_waave_active_api.
	 */
	private function call_waave_active_api() {
		$plugin_data = get_plugin_data( WAAVE_COMPLIANCE_MAIN_FILE );
		$body        = array(
			'version'             => $plugin_data['Version'],
			'ping_url'            => get_rest_url( null, 'waave-compliance/v1/version' ),
			'category_url'        => get_rest_url( null, 'waave-compliance/v1/products/categories' ),
			'menu_item_url'       => get_rest_url( null, 'waave-compliance/v1/products' ),
			'disclaimer_sync_url' => get_rest_url( null, 'waave-compliance/v1/disclaimer/sync' ),
			'venue_id'            => get_option( 'waave_compliance_venue_id' ),
			'password'            => get_option( 'waave_compliance_client_password' ),
		);

		$methods                 = get_option( 'waave_compliance_enabled_methods' );
		$body['payment_methods'] = $this->waave_enabled_payment_methods( $methods );

		$user          = wp_get_current_user();
		$body['actor'] = isset( $user->user_email ) ? $user->user_email : '';

		$url = WAAVE_PROD_URL . '/compliance/active';
		if ( 'yes' === get_option( 'waave_compliance_testmode' ) ) {
			$url = WAAVE_SANDBOX_URL . '/compliance/active';
		}

		$options = array(
			'body' => $body,
		);

		$request  = wp_remote_post( $url, $options );
		$response = json_decode( wp_remote_retrieve_body( $request ), true );

		if ( empty( $response['success'] ) ) {
			WC_Admin_Settings::add_error( $response['message'] );
		}
	}
}
